# -*- coding: utf-8 -*-
from flask_restful import Resource
from flask import make_response, jsonify
from openeo_grass_gis_driver.actinia_processing.actinia_interface import \
     ActiniaInterface
from openeo_grass_gis_driver.models.collection_schemas import \
     CollectionInformation, CollectionExtent
from openeo_grass_gis_driver.models.collection_schemas import \
     CollectionProperties, EOBands
from openeo_grass_gis_driver.local_collections import \
     get_local_collection
from osgeo import osr, ogr

__license__ = "Apache License, Version 2.0"
__author__ = "Sören Gebbert"
__copyright__ = "Copyright 2018, Sören Gebbert, mundialis"
__maintainer__ = "Soeren Gebbert"
__email__ = "soerengebbert@googlemail.com"

strds_example = {
    "aggregation_type": "None",
    "bottom": "0.0",
    "creation_time": "2016-08-11 16:44:29.756411",
    "creator": "soeren",
    "east": "75.5",
    "end_time": "2013-07-01 00:00:00",
    "ewres_max": "0.25",
    "ewres_min": "0.25",
    "granularity": "1 month",
    "id": "precipitation_1950_2013_monthly_mm@PERMANENT",
    "map_time": "interval",
    "mapset": "PERMANENT",
    "max_max": "1076.9",
    "max_min": "168.9",
    "min_max": "3.2",
    "min_min": "0.0",
    "modification_time": "2016-08-11 16:45:14.032432",
    "name": "precipitation_1950_2013_monthly_mm",
    "north": "75.5",
    "nsres_max": "0.25",
    "nsres_min": "0.25",
    "number_of_maps": "762",
    "raster_register": "raster_map_register_934719ed2b4841818386a6f9c5f11b09",
    "semantic_type": "mean",
    "south": "25.25",
    "start_time": "1950-01-01 00:00:00",
    "temporal_type": "absolute",
    "top": "0.0",
    "west": "-40.5"
}

raster_example = {
    "cells": "2025000",
    "cols": "1500",
    "comments": "\"r.proj input=\"ned03arcsec\" location=\"northcarolina_latlong\" mapset=\"\\helena\" output=\"elev_ned10m\" method=\"cubic\" resolution=10\"",
    "creator": "\"helena\"",
    "database": "/tmp/gisdbase_75bc0828",
    "datatype": "FCELL",
    "date": "\"Tue Nov  7 01:09:51 2006\"",
    "description": "\"generated by r.proj\"",
    "east": "645000",
    "ewres": "10",
    "location": "nc_spm_08",
    "map": "elevation",
    "mapset": "PERMANENT",
    "max": "156.3299",
    "min": "55.57879",
    "ncats": "255",
    "north": "228500",
    "nsres": "10",
    "rows": "1350",
    "source1": "\"\"",
    "source2": "\"\"",
    "south": "215000",
    "timestamp": "\"none\"",
    "title": "\"South-West Wake county: Elevation NED 10m\"",
    "units": "\"none\"",
    "vdatum": "\"none\"",
    "west": "630000"}


def coordinate_transform_extent_to_EPSG_4326(
        crs: str, extent: CollectionExtent):
    """Tranfor the extent coordinates to lat/lon

    :param crs:
    :param extent:
    :return:
    """

    source = osr.SpatialReference()
    source.ImportFromWkt(crs)
    source.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)

    target = osr.SpatialReference()
    target.ImportFromEPSG(4326)
    target.SetAxisMappingStrategy(osr.OAMS_TRADITIONAL_GIS_ORDER)

    transform = osr.CoordinateTransformation(source, target)

    lower_left = ogr.CreateGeometryFromWkt(
        f"POINT ({extent.spatial['bbox'][0][0]} {extent.spatial['bbox'][0][1]})")
    lower_left.Transform(transform)
    upper_right = ogr.CreateGeometryFromWkt(
        f"POINT ({extent.spatial['bbox'][0][2]} {extent.spatial['bbox'][0][3]})")
    upper_right.Transform(transform)

    a0 = lower_left.GetPoint()[0]
    a1 = lower_left.GetPoint()[1]
    a2 = upper_right.GetPoint()[0]
    a3 = upper_right.GetPoint()[1]

    extent.spatial['bbox'][0] = (a0, a1, a2, a3)
    return extent


class CollectionInformationResource(Resource):

    def __init__(self):
        self.iface = ActiniaInterface()

    def get(self, name):

        location, mapset, datatype, layer = self.iface.layer_def_to_components(
            name)

        # local collection registered in the openEO backend
        if location == "local":
            collection = get_local_collection(name=name)
            if collection is None:
                return make_response(
                    jsonify(
                        {
                            "id": "12345678",
                            "code": "CollectionNotFound",
                            "message": "Collection '%s' does not exist." %
                            (name),
                            "links": {}}),
                    500)

            # Not using CollectionInformation model here for now
            # as valid STAC collections comply.
            # Using it here might omit some properties
            # which are not modelled in this backend (e.g. assets)
            return make_response(collection, 200)

        # STAC collection registered in actinia
        if location == "stac":
            status_code, collection = self.iface.get_stac_collection(name=name)
            if status_code != 200:
                return make_response(
                    jsonify(
                        {
                            "id": "12345678",
                            "code": "Internal",
                            "message": "Server error: %s" %
                            (name),
                            "links": {}}),
                    500)

            # Not using CollectionInformation model here for now
            # as valid STAC collections comply.
            # Using it here might omit some properties
            # which are not modelled in this backend (e.g. assets)
            return make_response(collection, 200)

        # List strds maps from the GRASS location
        status_code, layer_data = self.iface.layer_info(layer_name=name)
        if status_code != 200:
            return make_response(
                jsonify(
                    {
                        "id": "12345678",
                        "code": "CollectionNotFound",
                        "message": "Collection '%s' does not exist." %
                        (name),
                        "links": {}}),
                404)

        # Get the projection from the GRASS mapset
        status_code, mapset_info = self.iface.mapset_info(
            location=location, mapset=mapset)
        if status_code != 200:
            return make_response(
                jsonify(
                    {
                        "id": "12345678",
                        "code": "Internal",
                        "message": "Server error: %s" %
                        (mapset_info),
                        "links": {}}),
                500)

        extent = CollectionExtent(
            spatial=(
                float(
                    layer_data["west"]), float(
                    layer_data["south"]), float(
                    layer_data["east"]), float(
                        layer_data["north"])), temporal=(
                            "1900-01-01T00:00:00", "2100-01-01T00:00:00"))

        title = "Raster dataset"
        bands = []
        dimensions = {"x": {
            "type": "spatial",
            "axis": "x",
            "extent": [layer_data["west"], layer_data["east"]],
            "reference_system": mapset_info["projection"]
        },
            "y": {
            "type": "spatial",
            "axis": "y",
            "extent": [layer_data["south"], layer_data["north"]],
            "reference_system": mapset_info["projection"]
        },
        }
        platform = "unknown"
        instrument = "unknown"
        if datatype.lower() == "strds":
            title = "Space time raster dataset"

            start_time = layer_data["start_time"]
            end_time = layer_data["end_time"]

            if start_time:
                start_time = start_time.replace(
                    " ",
                    "T").replace(
                    "'",
                    "").replace(
                    '"',
                    '')

            if end_time:
                end_time = end_time.replace(
                    " ",
                    "T").replace(
                    "'",
                    "").replace(
                    '"',
                    '')

            dimensions['t'] = {"type": "temporal",
                               "extent": [start_time, end_time]
                               }

            extent = CollectionExtent(
                spatial=(
                    float(
                        layer_data["west"]), float(
                        layer_data["south"]), float(
                        layer_data["east"]), float(
                        layer_data["north"])), temporal=(
                            start_time, end_time))

            if "semantic_labels" in layer_data:
                bandlist = layer_data["semantic_labels"].split(',')
                dimensions['bands'] = {"type": "bands",
                                       "values": bandlist
                                       }
                for bandname in bandlist:
                    # not so nice, better use different name and common_name
                    # waiting for GRASS GIS
                    bands.append(EOBands(name=bandname, common_name=bandname))

                # get platform and sensor
                # see
                # https://github.com/radiantearth/stac-spec/blob/master/item-spec/common-metadata.md#platform
                # https://github.com/radiantearth/stac-spec/blob/master/item-spec/common-metadata.md#instruments
                if "_" in bandlist[0]:
                    sensor_abbr = bandlist[0].split('_')[0]
                    if sensor_abbr == "L5":
                        platform = "landsat-5"
                        instrument = "tm, mss"
                    elif sensor_abbr == "L7":
                        platform = "landsat-7"
                        instrument = "etm+"
                    elif sensor_abbr == "L8":
                        platform = "landsat-8"
                        instrument = "oli, trs"
                    elif sensor_abbr == "S1":
                        platform = "sentinel-1"
                        instrument = "c-sar"
                    elif sensor_abbr == "S2":
                        platform = "sentinel-2"
                        instrument = "msi"

        if datatype.lower() == "vector":
            title = "Vector dataset"

        description = "GRASS GIS location/mapset path: /%s/%s" % (
            location, mapset)
        crs = mapset_info["projection"]

        coordinate_transform_extent_to_EPSG_4326(crs=crs, extent=extent)

        # GRASS / actinia do not yet report platform and instrument
        properties = (CollectionProperties(eo_platform=platform,
                                           eo_instrument=instrument,
                                           eo_bands=bands))

        ci = CollectionInformation(id=name, title=title,
                                   description=description,
                                   extent=extent,
                                   properties=properties,
                                   dimensions=dimensions)

        return ci.as_response(http_status=200)
